@@ -1,5 +1,5 @@
 """
-Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - January 22, 2021
+Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - February 28, 2022
 """
 
 import time
@@ -24,29 +24,26 @@ def ProblemSolving(DirName, CaseName, SolverName, OptModel, mTEPES, pIndLogConso
     mTEPES.n  = Set(initialize=mTEPES.nn,  ordered=True, doc='load levels', filter=lambda mTEPES,nn : nn  in                mTEPES.pDuration        )
 
     #%% solving the problem
-    # Solver = SolverFactory(SolverName, solver_io='mps')                                      # select solver
     Solver = SolverFactory(SolverName)                                                         # select solver
     if SolverName == 'gurobi':
         Solver.options['LogFile'       ] = _path+'/openTEPES_'+CaseName+'.log'
         # Solver.options['IISFile'     ] = _path+'/openTEPES_'+CaseName+'.ilp'               # should be uncommented to show results of IIS
         Solver.options['Method'        ] = 2                                                 # barrier method
+        Solver.options['MIPFocus'      ] = 1
         Solver.options['Presolve'      ] = 2
-        Solver.options['Crossover'     ] = 0
         Solver.options['RINS'          ] = 100
         # Solver.options['BarConvTol'    ] = 1e-9
         # Solver.options['BarQCPConvTol' ] = 0.025
         Solver.options['MIPGap'        ] = 0.01
         Solver.options['Threads'       ] = int((psutil.cpu_count(logical=True) + psutil.cpu_count(logical=False))/2)
-        Solver.options['TimeLimit'     ] =    9600
-        Solver.options['IterationLimit'] = 9600000
+        Solver.options['TimeLimit'     ] =    10800
+        Solver.options['IterationLimit'] = 10800000
     if mTEPES.pIndBinGenInvest()*len(mTEPES.gc) + mTEPES.pIndBinGenRetire()*len(mTEPES.gd) + mTEPES.pIndBinNetInvest()*len(mTEPES.lc) + mTEPES.pIndBinGenOperat()*len(mTEPES.nr) + mTEPES.pIndBinLineCommit()*len(mTEPES.la) + len(mTEPES.g2g) == 0:
-        if SolverName == 'gurobi' or SolverName == 'mosek':
-            Solver.options['relax_integrality'] =  1  # introduced to show results of the dual variables
-            Solver.options['Crossover'        ] = -1
         OptModel.dual = Suffix(direction=Suffix.IMPORT)
         OptModel.rc   = Suffix(direction=Suffix.IMPORT)
-    SolverResults = Solver.solve(OptModel, tee=True , report_timing=True )               # tee=True displays the log of the solver
-    assert str(SolverResults.solver.termination_condition) == 'optimal'
+    SolverResults = Solver.solve(OptModel, tee=True , report_timing=True )             # tee=True displays the log of the solver
+    print('Termination Condition: ', SolverResults.solver.termination_condition)
+    assert (str(SolverResults.solver.termination_condition) == 'optimal' or str(SolverResults.solver.termination_condition) == 'maxTimeLimit' or str(SolverResults.solver.termination_condition) == 'maxIterations')
     SolverResults.write()                                                              # summary of the solver results
 
     #%% fix values of somes variables to get duals and solve it again
@@ -86,20 +83,19 @@ def ProblemSolving(DirName, CaseName, SolverName, OptModel, mTEPES, pIndLogConso
         if len(mTEPES.g2g):
             for nr in mTEPES.nr:
                 if sum(1 for g in mTEPES.nr if (nr,g) in mTEPES.g2g or (g,nr) in mTEPES.g2g):
-                    OptModel.vMaxCommitment[nr].fix(OptModel.vMaxCommitment[nr]())
-        if SolverName == 'gurobi' or SolverName == 'mosek':
-            Solver.options['relax_integrality'] =  1  # introduced to show results of the dual variables
-            Solver.options['Crossover'        ] = -1
+                    OptModel.vMaxCommitment[nr].fix(round(OptModel.vMaxCommitment[nr]()))
         OptModel.dual = Suffix(direction=Suffix.IMPORT)
         OptModel.rc   = Suffix(direction=Suffix.IMPORT)
-        SolverResults = Solver.solve(OptModel, tee=True , report_timing=True , warmstart=True)   # tee=True displays the log of the solver
+        SolverResults = Solver.solve(OptModel, tee=True, report_timing=True, warmstart=True)   # tee=True displays the log of the solver
         SolverResults.write()                                                                  # summary of the solver results
 
     SolvingTime = time.time() - StartTime
     print('Solution time                          ... ', round(SolvingTime), 's')
     print('Total system      cost [MEUR]              ', OptModel.eTotalTCost.expr())
-    print('Total investment  cost [MEUR]              ', OptModel.vTotalFCost())
-    print('Total generation  cost [MEUR]              ', sum(mTEPES.pScenProb[sc] * OptModel.vTotalGCost[sc,p,n]() for sc,p,n in mTEPES.sc*mTEPES.p*mTEPES.n))
-    print('Total consumption cost [MEUR]              ', sum(mTEPES.pScenProb[sc] * OptModel.vTotalCCost[sc,p,n]() for sc,p,n in mTEPES.sc*mTEPES.p*mTEPES.n))
-    print('Total emission    cost [MEUR]              ', sum(mTEPES.pScenProb[sc] * OptModel.vTotalECost[sc,p,n]() for sc,p,n in mTEPES.sc*mTEPES.p*mTEPES.n))
-    print('Total reliability cost [MEUR]              ', sum(mTEPES.pScenProb[sc] * OptModel.vTotalRCost[sc,p,n]() for sc,p,n in mTEPES.sc*mTEPES.p*mTEPES.n))
+    print('Total Genr invest cost [MEUR]              ', sum(mTEPES.pGenInvestCost[gc] * OptModel.vGenerationInvest[gc]() for gc     in mTEPES.gc                  ))
+    print('Total Retr invest cost [MEUR]              ', sum(mTEPES.pGenRetireCost[gd] * OptModel.vGenerationRetire[gd]() for gd     in mTEPES.gd                  ))
+    print('Total Line invest cost [MEUR]              ', sum(mTEPES.pNetFixedCost [lc] * OptModel.vNetworkInvest   [lc]() for lc     in mTEPES.lc                  ))
+    print('Total generation  cost [MEUR]              ', sum(mTEPES.pScenProb     [sc] * OptModel.vTotalGCost  [sc,p,n]() for sc,p,n in mTEPES.sc*mTEPES.p*mTEPES.n))
+    print('Total consumption cost [MEUR]              ', sum(mTEPES.pScenProb     [sc] * OptModel.vTotalCCost  [sc,p,n]() for sc,p,n in mTEPES.sc*mTEPES.p*mTEPES.n))
+    print('Total emission    cost [MEUR]              ', sum(mTEPES.pScenProb     [sc] * OptModel.vTotalECost  [sc,p,n]() for sc,p,n in mTEPES.sc*mTEPES.p*mTEPES.n))
+    print('Total reliability cost [MEUR]              ', sum(mTEPES.pScenProb     [sc] * OptModel.vTotalRCost  [sc,p,n]() for sc,p,n in mTEPES.sc*mTEPES.p*mTEPES.n))
